/***************************************************************************
* Author:
*	-Kev	(mailto:kevin_bertram@hp.com)
*
*     Copyright (c) 2001 Hewlett Packard Pty. Ltd., All Rights Reserved.
*     
*     This program is free software; you can redistribute it and/or 
*     modify it under the terms of the GNU General Public License as 
*     published by the Free Software Foundation; either version 2 of 
*     the License, or (at your option) any later version.
* 
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*     GNU General Public License for more details.
* 
*     You should have received a copy of the GNU General Public License 
*     along with this program; if not, write to the Free Software 
*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
*     MA 02111-1307 USA
*
* FILE:
*   files.c
*
****************************************************************************
* Purpose:
*
*   Automatic PPP connection using IrNET. This program listens to events
*	in /dev/irnet and creates a PPP connection if it detects a connection
*	request
*
*   The original concept was derived from:
*
*		Luiz Magalhaes - HPL '00 (luiz_magalhaes@hp.com)
*		Jean Tourrilhes - HPL '00 <jt@hpl.hp.com>
*
*	The node "/dev/irnet" is at major 10, minor 187.
*
*	This module manages the external files, most notibly /etc/hosts file.
****************************************************************************
* Revision History:
*
* Version 0.1.8			5-October-2001		-Kev
*	(1)	First split this module off from the main code body.
*****************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<signal.h>
#include	<unistd.h>
#include	<ctype.h>
#include	<stdarg.h>
#include	<sys/types.h>
#include	<sys/time.h>

/**** Read in locally defined includes for this module ****/

#include	"irnetd.h"
#include	"getopt.h"
#include	"files.h"



/***************************************************************************
 Add the given "command" string  as an entry to the /etc/hosts file.

 The format of the control string "command" is :

	"Key_word from 0A123456 (nickname)"

 Returns: 0
****************************************************************************/

int	AddToHosts( char * command )
{
	 int	i;
	 int	adrs;
	char *	ptr;
	char	node[40];
	char	ipadrs[40];
	char	nickname[40];
	char	entry[80];

	Dprintf( 3, "AddToHosts(): Adding %s\n", command );

/**** Get nodes name ****/

	ptr = command;
	memset( node, 0, sizeof(node) );

	while ((*ptr) && (*ptr != ' ')) ptr++;
	while ((*ptr) && (*ptr == ' ')) ptr++;

	if (strncmp( ptr, "from", 4 ) == 0)
	{
		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;
	}

	for (i=0; (i < sizeof(node)) && (*ptr) && (isxdigit(*ptr)); i++, ptr++)
	{
		node[i] = *ptr;
	}

/**** Get the nodes nickname ****/

	ptr = command;
	memset( nickname, 0, sizeof(nickname) );

	while ((*ptr) && (*ptr != '(')) ptr++;

	if (*ptr == '(')
	{
		ptr++;

		for (i=0;(*ptr) && (*ptr != ')'); i++, ptr++ )
		{
			nickname[i] = *ptr;
		}
	}
	else
	{
		strcpy( nickname, "HP Jornada" );
	}

/**** Construct the new Hosts entry line ****/

	i = FindConnectEntry( node );

	sscanf( node, "%x", &adrs );
	adrs = PruneAddress( adrs );

	sprintf( ipadrs, "%d.%d.%d.%d",
	         (adrs >> 24) & 0xFF,
			 (adrs >> 16) & 0xFF,
			 (adrs >> 8)  & 0xFF,
			 (adrs & 0xFF) );

	sprintf( entry, "%s\t%s%d\t\t# IrDA PPP connected device\n",
			 ipadrs,
			 HOSTS_NAME_STUB,
			 i );

/**** Add this entry to the hosts file ****/

	AddEntryToHosts( ipadrs, entry );

	Dprintf( 2, "AddToHosts(): host line added \"%s\"\n", entry );

	return( 0 );
}


/***************************************************************************
 Remove this device from the /etc/hosts file.
 The format of the control string "command" is :

	"Key_word from 0A123456 (nickname)"

 Returns: 0
****************************************************************************/

int	RemoveFromHosts( char * command )
{
	 int	i;
	 int	adrs = 0;
	char *	ptr;
	char	node[40];
	char    ipadrs[20];
	char	entry[80];

	Dprintf( 2, "RemoveFromHosts(): Removing %s\n", command );

/**** Get nodes name ****/

	ptr = command;
	memset( node, 0, sizeof(node) );

	while ((*ptr) && (*ptr != ' ')) ptr++;
	while ((*ptr) && (*ptr == ' ')) ptr++;

	if (strncmp( ptr, "from", 4 ) == 0)
	{
		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;
	}

	for (i=0; (i < sizeof(node)) && (*ptr) && (isxdigit(*ptr)); i++, ptr++)
	{
		node[i] = *ptr;
	}

/**** Convert node name to IP address ****/

	sscanf( node, "%x", &adrs );
	adrs = PruneAddress( adrs );

	sprintf( ipadrs, "%d.%d.%d.%d",
	         (adrs >> 24) & 0xFF,
			 (adrs >> 16) & 0xFF,
			 (adrs >> 8)  & 0xFF,
			 (adrs & 0xFF) );

/**** Construct the new Hosts entry line ****/

	i = FindConnectEntry( node );

	sscanf( node, "%x", &adrs );
	adrs = (adrs & 0x00FFFFFF) | 0x0A000000;

	sprintf( entry, "%s\t%s%d\t\t# IrDA PPP connected device\n",
			 ipadrs,
			 HOSTS_NAME_STUB,
			 i );

/**** Remove this entry from the hosts file ****/

	RemoveEntryFromHosts( ipadrs );

	Dprintf( 2, "RemoveFromHosts(): host line \"%s\"\n", ipadrs );

	return( 0 );
}


/***************************************************************************
 This function walks through the hosts file (/etc/hosts) and adds the given
 entry to the file. The entry must conform to the hosts file layout like
 this:
           ip_adrs    alias0     alias1     aliasN

 Returns:	0    	-	Entry added successfully.
			FAILED	-	Failed to add entry to file.
****************************************************************************/

int	AddEntryToHosts( char * ipadrs, char * new_entry )
{
	 int	n;
	 int	return_code = FAILED;
	FILE *	fh = NULL;

	if (gFlags.Mode & MODE_HOSTS_FILE)
	{
		Dprintf( 2, "AddEntryToHosts(): Adding hosts entry \"%s\"\n", new_entry );

		n = FindEntryInHosts( ipadrs );

		if (n == FAILED)
		{
			fh = fopen( FILE_HOSTS, "a" );

			if (fh)
			{
				fseek( fh, 0L, SEEK_END );
				fputs( new_entry, fh );
				fclose( fh );
			}
		}
	}
	else
	{
		return_code = 0;
	}

/**** Cleanup and return status to caller ****/

	return( return_code );
}


/***************************************************************************
 This function walks through the hosts file (/etc/hosts) and removes the
 given entry to the file. The entry must, therefore conform to the hosts
 file layout like this:

           ip_adrs    alias0     alias1     aliasN

 Returns:	0    	-	Entry found and removed successfully.
			FAILED	-	Failed to find and remove the entry.
****************************************************************************/

int	RemoveEntryFromHosts( char * ipadrs )
{
	 int	n;
	 int	count;
	 int	return_code = FAILED;
	FILE *	srcfh = NULL;
	FILE *	dstfh = NULL;
	char	line[256];

	if (gFlags.Mode & MODE_HOSTS_FILE)
	{
		Dprintf( 2, "RemoveEntryFromHosts(): Removing hosts entry \"%s\"\n", ipadrs );

		n = FindEntryInHosts( ipadrs );

		if (n != FAILED)
		{
			srcfh = fopen( FILE_HOSTS, "r" );
			dstfh = fopen( FILE_TMP,   "w" );

			if (srcfh && dstfh)
			{
				for (count=0; (feof(srcfh) == 0) && (count < ETC_HOST_MAXLINES); count++ )
				{
					memset( line, 0, sizeof(line) );
					fgets( line, sizeof(line), srcfh );

					if (strlen(line) > 0) 
					{
						Dprintf( 3, "RemoveEntryFromHosts(): < Line[%d]=\"%s\"\n", count, line );

						if (n != count)
						{
							fputs( line, dstfh );
						}
					}
				}

				fclose( srcfh );
				fclose( dstfh );

/**** Replace old hosts file with the new edited one ****/

				srcfh = fopen( FILE_TMP,   "r" );
				dstfh = fopen( FILE_HOSTS, "w" );

				if (dstfh && srcfh)
				{
					for (count=0; (feof(srcfh) == 0) && (count < ETC_HOST_MAXLINES); count++ )
					{
						memset( line, 0, sizeof(line) );
						fgets( line, sizeof(line), srcfh );

						if (strlen(line) > 0) 
						{
							fputs( line, dstfh );
							Dprintf( 3, "RemoveEntryFromHosts(): > Line[%d]=\"%s\"\n", count, line );
						}
					}
					
					return_code = 0;
					fclose( srcfh );
					fclose( dstfh );

					srcfh = NULL;
					dstfh = NULL;
				}
			}
		}
	}
	else
	{
		return_code = 0;
	}

/**** Cleanup and return status to caller ****/

	if (srcfh) fclose( srcfh );
	if (dstfh) fclose( dstfh );

	return( return_code );
}


/***************************************************************************
 This function searches the hosts file (/etc/hosts) for an entry that matches
 the given string. If it is found then the line number of it is returned
 else FAILED is returned.

 Returns: >= 0		Line number of entry in file.
		  FAILED	Entry does not exist in hosts file.
****************************************************************************/

int	FindEntryInHosts( char * ipadrs )
{
	 int	count;
	 int	return_code = FAILED;
	FILE *	fh;
	char	line[256];

/**** Try to open the file for reading ****/

	fh = fopen( FILE_HOSTS, "r" );

	if (fh)
	{
		for (count=0; (count < ETC_HOST_MAXLINES) && (feof(fh) == 0); count++)
		{
			memset( line, 0, sizeof(line) );
			fgets( line, sizeof(line), fh );

			if (strlen( line ) > 0)
			{
				if (strncmp( line, ipadrs, strlen(ipadrs) ) == 0)
				{
					return_code = count;
					break;
				}
			}
		}

		fclose( fh );

		Dprintf( 3, "FindEntryInHosts(): Found hosts entry at line %d\n", return_code );
	}
	else
	{
		Dprintf( 1, "FindEntryInHosts(): Failed to open hosts file \"%s\"\n", FILE_HOSTS );
	}

	return( return_code );
}


/***************************************************************************
 This function searches the hosts file (/etc/hosts) for an entry name that
 matches the given hostname. If there is a line that matches the it's line 
 number of it is returned else FAILED is returned.

 Returns: >= 0		Line number of entry in file.
		  FAILED	Entry does not exist in hosts file.
****************************************************************************/

int	FindNameInHosts( char * name )
{
	 int	count;
	 int	return_code = FAILED;
	FILE *	fh;
	char	line[256];

/**** Try to open the file for reading ****/

	fh = fopen( FILE_HOSTS, "r" );

	if (fh)
	{
		for (count=0; (count < ETC_HOST_MAXLINES) && (feof(fh) == 0); count++)
		{
			memset( line, 0, sizeof(line) );
			fgets( line, sizeof(line), fh );

			if (strlen( line ) > 0)
			{
				if (strncmp( line, name, strlen(name) ) == 0)
				{
					return_code = count;
					break;
				}
			}
		}

		fclose( fh );

		Dprintf( 3, "FindNameInHosts(): Found hosts entry at line %d\n", return_code );
	}
	else
	{
		Dprintf( 1, "FindNameInHosts(): Failed to open hosts file \"%s\"\n", FILE_HOSTS );
	}

	return( return_code );
}


/*************************************************************************
 The job of this function is to search out all references to IR devices
 in the /etc/hosts file and remove them. This is normally needed when the
 program exits and needs to remove all evidence of itself.

 Returns: Nothing. /etc/hosts file should be clean of IR device references.
**************************************************************************/

void	CleanHostFile( void )
{
	 int	count;
	FILE *	srcfh = NULL;
	FILE *	dstfh = NULL;
	char *	ptr;
	char	line[256];

	Dprintf( 2, "CleanHostFile(): Cleaning host file of IR device presence\n" );

	srcfh = fopen( FILE_HOSTS, "r" );
	dstfh = fopen( FILE_TMP,   "w" );

	if (srcfh && dstfh)
	{
		for (count=0; (feof(srcfh) == 0) && (count < ETC_HOST_MAXLINES); count++ )
		{
			memset( line, 0, sizeof(line) );
			fgets( line, sizeof(line), srcfh );

			if (strlen(line) > 0) 
			{
				ptr = line;

				while (*ptr && strncmp(ptr, HOSTS_NAME_STUB, strlen(HOSTS_NAME_STUB)) ) ptr++;

				if (*ptr == '\0')
				{
					fputs( line, dstfh );
				}
				else
				{
					Dprintf( 3, "CleanHostFile(): Deleting Line[%d]=\"%s\"\n", count, line );
				}
			}
		}

		fclose( srcfh );
		fclose( dstfh );

/**** Replace old hosts file with the new edited one ****/

		srcfh = fopen( FILE_TMP,   "r" );
		dstfh = fopen( FILE_HOSTS, "w" );

		if (dstfh && srcfh)
		{
			for (count=0; (feof(srcfh) == 0) && (count < ETC_HOST_MAXLINES); count++ )
			{
				memset( line, 0, sizeof(line) );
				fgets( line, sizeof(line), srcfh );

				if (strlen(line) > 0) 
				{
					fputs( line, dstfh );
					Dprintf( 3, "CleanHostFile(): Retained Line[%d]=\"%s\"\n", count, line );
				}
			}
			
			fclose( srcfh );
			fclose( dstfh );

			srcfh = NULL;
			dstfh = NULL;
		}
	}

/**** Cleanup and return status to caller ****/

	if (srcfh) fclose( srcfh );
	if (dstfh) fclose( dstfh );
}


/*************************************************************************
 This functions reads out the IrLAP MAC address of this machine and uses
 it as the IP address of this PPP interface. This method assumes that
 the IrDA->MAC->10.x.x.x IP address bind/mapping has been enabled in the
 kernel. If not the this function will return an error.

 To test that the kernel is producing the correct IP->MAC address a test
 is performed on the upper byte of the address. It should be equal to
 10 = 0x0Axxxxxx.

 Example of text read from proc entry:

 irlap0 state: LAP_NDM
  device name: irda0, hardware name: 0a219cea-ir
  caddr: 0xea, saddr: 0xa219cea, daddr: 0x000000
  win size: 0, win: 0, line capacity: 0, bytes left: 0
  tx queue len: 0 win queue len: 0 rbusy: FALSE mbusy: FALSE
  retrans: 0 vs: 0 vr: 0 va: 0
  qos   bps     maxtt   dsize   winsize addbofs mintt   ldisc   comp
  tx    9600    0       64      1       12      0       0
  rx    9600    0       64      1       12      0       0

 Returns:	TRUE:	Got address and here it is.
			FAILED:	No address to read because IRDA interface not present
**************************************************************************/

int	GetIpAddress( unsigned int * adrs )
{
	FILE *	fh;
	 int	n;
	 int	count;
	 int	return_code = FAILED;
    char *	str_ptr;
    char	buffer[BUFFER_SIZE];

/**** Try to open the proc entry so we can read our SADDRS (MAC) ****/

	if (gFlags.Myaddress != -1)
	{
		*adrs = gFlags.Myaddress;
		return_code = TRUE;
	}
	else
	{
		fh = fopen( PROC_IRLAP, "r" );

		if (fh)
		{
/**** Now extract the SADDR value from the proc entry ****/

			count = fread( buffer, 1, BUFFER_SIZE, fh );

			if (count > 0)
			{
				str_ptr = strstr(buffer, "saddr: ");

				if (str_ptr == NULL)
				{
					str_ptr = strstr(buffer, "SADDR: ");
				}

				if (str_ptr == NULL)
				{
					return_code = FAILED;
				}
				else
				{
					for (n=0; (n < 20) && (*str_ptr != 'x'); n++, str_ptr++ );

					str_ptr++;

/**** Convert hexadecimal string to integer ****/

					n = sscanf( str_ptr, "%x", adrs );
					*adrs = PruneAddress( *adrs );

					return_code = TRUE;
				}
			}

/**** Clean up and return answer to caller ****/

			fclose( fh );
		}
	}

	return( return_code );
}


/*************************************************************************
 Use this function to ensure that a given IP address lines up with the
 10.x.x.x subnet.

 Returns: fixed IP address.
**************************************************************************/

unsigned int PruneAddress( unsigned int adrs )
{
	return( (adrs & 0x00FFFFFF) | 0x0A000000 );
}


/*************************************************************************
 This function can be used to turn on or off the polling mode of IrLAP.
 If it is turned on then the device will be asked to continuosly seek out
 all new devices that come into IrDA range with this one.

 If it is turned off then this device listens for a Master and other
 conversatins.

 Returns:	0
**************************************************************************/

int	MasterMode( int mode )
{
	FILE *	fh;
    char	buffer[4];

	memset( buffer, 0, sizeof(buffer) );

	fh = fopen( PROC_MODE, "w" );

	if (fh)
	{
		if (mode)
		{
			buffer[0] = '1';
			fwrite( buffer, 1, 1, fh );
			printf( "MasterMode(): Device is set to IrDA Master\n" );
		}
		else
		{
			buffer[0] = '0';
			fwrite( buffer, 1, 1, fh );
			printf( "MasterMode(): Device is set to IrDA Slave\n" );
		}

		fclose( fh );
	}
	else
	{
		printf( "%s: Error - could not open proc entry \"%s\"\n",
		        PROGRAM_NAME, PROC_MODE );
	}

	return( 0 );
}


/*************************************************************************
 This function looks at the discovery log for IrLAP and works out if there
 is another device present. If there are no entries in the log then it is
 assumed that there no one to talk to or there is no master.

 Handling what happens next will be the job of the caller.

 Returns:	> 0		- Count of devices listed.
			FALSE	- There are no connections visible.
			FAILED	- IrDA network interface is not present.
**************************************************************************/

int	IsIrdaDevices( void )
{
	FILE *	fh;
	 int	len;
	 int	size;
	 int	count = 0;
	 int	return_code = FAILED;
	char *	string = "nickname:";
    char	buffer[4096];

	memset( buffer, 0, sizeof(buffer) );

	fh = fopen( PROC_DISCOVERY, "r" );

	if (fh)
	{
		len = 0;
		size = fread( buffer, 1, sizeof(buffer), fh );


		while ((buffer[len]) && (len < size))
		{
			if (buffer[len] == 'n')
			{
				if (strncmp( &buffer[len], string, strlen(string) ) == 0)
				{
					count++;
					len += strlen(string);
				}
				else
				{
					len++;
				}
			}
			else
			{
				len++;
			}
		}

		if (count == 0)	return_code = 0;
		else			return_code = count;

		fflush( fh );
		fclose( fh );
	}

	return( return_code );
}


/*************************************************************************\
 This function reads the "mode" control proc entry for the IrDA network
 interface and reports if this devices has been set to "Master" mode or
 not.

 Returns:	TRUE	- The device is a master
			FALSE	- This device is a Slave
			FAILED	- IrDA network interface is not present.
**************************************************************************/

int	IsMaster( void )
{
	 int	return_code = FAILED;
	FILE *	fh;
    char	buffer[4];

	memset( buffer, 0, sizeof(buffer) );

	fh = fopen( PROC_MODE, "r" );

	if (fh)
	{
		fread( buffer, 1, 1, fh );
		fclose( fh );

		switch (buffer[0])
		{
			case ('0'):	return_code = FALSE;	break;
			case ('1'):	return_code = TRUE;		break;
			default:	return_code = FAILED;	break;
		}
	}

	return( return_code );
}




/*********************************  EOF  **********************************/

