
/**********************************************************************
  Module:	getopt.o
* Author:
*	-Kev	(mailto:kevin_bertram@hp.com)
*
*     Copyright (c) 2001 Hewlett Packard Pty. Ltd., All Rights Reserved.
*     
*     This program is free software; you can redistribute it and/or 
*     modify it under the terms of the GNU General Public License as 
*     published by the Free Software Foundation; either version 2 of 
*     the License, or (at your option) any later version.
* 
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*     GNU General Public License for more details.
* 
*     You should have received a copy of the GNU General Public License 
*     along with this program; if not, write to the Free Software 
*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
*     MA 02111-1307 USA
*
***********************************************************************
* Description:
*	This module is used to search the command line arguments and
*	decompile them into a structure from which the program will be
*	driven from.
*
*	The controlling structure "gFlags" is first initialized to a default
*	behaviour the program should assume and then that behaviour will be
*	modified by each new command line switch found.
*
*  Note: The gFlags global structure is required by all modules and
*        controls the general actions of all modules and functions.
*
***********************************************************************/

#ifndef	GETOPT_H_INCLUDED
#define	GETOPT_H_INCLUDED


/**** Use this PPPD command line to start a connection almost permenantly 
      while using LCP pings to determine link end-to-end presence. When the
	  link is broken PPPD drops. The problem with this method is that the
	  LCP pings prevent communication to any other IR device because the
	  link is always busy between here and there. That means this is only 
	  a one user (connection) solution. IrLAP comms only switchs to another
	  device when the current circuit goes idle and it drops back to 9600bps
	  negotiation speed. This is a limitation of the IrDA specs not PPPD ****/

#define	PPP_CMD1	"pppd /dev/irnet 9600 connect-delay 0 noauth local novj novjccomp lcp-echo-interval 2 lcp-echo-failure 5 nopcomp nobsdcomp"

/**** Use this PPPD command line  to start a connection to another device
      where the connection will be monitored by a watchdog idle timer. If
	  there are no IP packets passed over the link then it will timeout
	  and drop the connection. This is a better solution if more than one
	  device is to be communicated to. ****/

#define	PPP_CMD2	"pppd /dev/irnet 9600 connect-delay 0 noauth local nopcomp novj novjccomp"


/**** Define the global controls structure ****/

typedef struct
{
	  int	Mode;				/* Mode program should act in */
	  int	Idle;				/* Idle timeout count, in seconds, for PPPD */
	  int	Myaddress;			/* Force IP address to use over IrDA MAC address */
	  int	Debug_level;		/* Used to represent the message reporting level */
	  int	Show_usage;			/* If True show user usage message */
	  int	Show_version;		/* When true forces program to show its version number */
	 char	Daddr[40];			/* IrLAP device node address to connect to */
	 char 	Command[256];		/* Points to image of catenated command line */
	 char	Program[256];		/* Name of this program */
	 char	Pppcmd[256];		/* PPPD command line */
} gFlagsType;


/**** These functions can be used by other modules ****/

	extern   int	GetOpt( int, char ** );
	extern  void	ShowSoftUsage( void );


/**** These global variables can be accessed by all modules ****/

extern gFlagsType	gFlags;		/* Controls all program operations */

#endif


/****************************  EOF  ***********************************/


