
/***************************************************************************
* Author:
*	-Kev	(mailto:kevin_bertram@hp.com)
*
*     Copyright (c) 2001 Hewlett Packard Pty. Ltd., All Rights Reserved.
*     
*     This program is free software; you can redistribute it and/or 
*     modify it under the terms of the GNU General Public License as 
*     published by the Free Software Foundation; either version 2 of 
*     the License, or (at your option) any later version.
* 
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*     GNU General Public License for more details.
* 
*     You should have received a copy of the GNU General Public License 
*     along with this program; if not, write to the Free Software 
*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
*     MA 02111-1307 USA
*
* FILE:
*   irnetd.h
*
****************************************************************************
* Purpose:
*
*   Automatic PPP connection using IrNET. This program listens to events
*	in /dev/irnet and creates a PPP connection if it detects a connection
*	request
*
*   The original concept was derived from:
*
*		Luiz Magalhaes - HPL '00 (luiz_magalhaes@hp.com)
*		Jean Tourrilhes - HPL '00 <jt@hpl.hp.com>
*
*	The node "/dev/irnet" is at major 10, minor 187.
*
*	This module controls the whole application.
*****************************************************************************/

#ifndef	IRNETD_H_INCLUDED
#define	IRNETD_H_INCLUDED

#ifndef	FAILED
#   define	FAILED	-1
#endif

#ifndef	SUCCESS
#   define	SUCCESS	0
#endif

#ifndef	FALSE
#   define	FALSE	0
#endif

#ifndef	TRUE
#   define	TRUE	1
#endif

/**** Define everything used to control the behaviour of this program ****/

#define	ON					TRUE
#define	OFF					FALSE

#define	ETC_HOST_MAXLINES	20						/* Maximum allowed lines in hosts file */
#define	MAX_CONNECTIONS		10						/* Maximum number of connections allowed */
#define BUFFER_SIZE			1024					/* Size of buffer used to read proc entries */
#define	IDLE_PERIOD			360						/* Period of idleness disconnect count */

#define	PROGRAM_NAME		"irnetd"				/* Default name of this program */
#define	PROGRAM_REVISION	"1.0.0"					/* Software revision level */
#define	HOSTS_NAME_STUB		"irdevice"				/* /etc/hosts name to use for each new device */
#define	FILE_HOSTS			"/etc/hosts"			/* Name of name lookup hosts file */
#define	FILE_TMP			"/tmp/irnetd.tmp"		/* Name of temp file to use for hosts management */
#define	DEVICE_IRNET		"/dev/irnet"			/* IrNET device node */
#define	PROC_IRLAP			"/proc/net/irda/irlap"	/* Proc entry that describes IrDA interface */
#define	PROC_MODE			"/proc/sys/net/irda/discovery"
													/* This entry allows command to be given to IrLAP */
#define	PROC_DISCOVERY		"/proc/net/irda/discovery"
													/* List of discovered devices */

#define	CMD_DISCOVERED		"Discovered"			/* IrNET discovered command */
#define	CMD_EXPIRED			"Expired"				/* IrNET Expired command */
#define	CMD_FOUND			"Found"					/* IrNET Found command */
#define	CMD_REQUEST			"Request"				/* IrNET request for connection command */
#define	CMD_DISCONNECTION	"Disconnection"			/* IrNET reports PPP disconect event */
#define	CMD_CONNECTION		"Connection"			/* IrNET reports PPP successful connection */
#define	CMD_BLOCKED			"Blocked"				/* IrNET reports IrDA channel lost or blocked */

/**** These define the gFlags.Mode control flag ****/

#define	MODE_SERVER_ONLY		0x0001		/* Program only serves connection requests */
#define	MODE_MASTER_ON			0x0002		/* Turn on  master mode */
#define	MODE_MASTER_OFF			0x0004		/* Turn off master mode */
#define	MODE_CONNECT			0x0008		/* Initiate a connection to remote */
#define	MODE_DEVELOPMENT		0x0020		/* Sets development mode running */
#define	MODE_TIMER_ONLY			0x0040		/* Used PPPD timeout only to monitor link */
#define	MODE_HOSTS_FILE			0x0080		/* Flags method to treate hosts file with */
#define	MODE_DAEMON				0x0100		/* Run as daemon when set */

/**** This enumeration defines the mode this program will behave as ****/

enum
{
	MODE_DUMP,					/* Act as a memory dump program */
	MODE_EDIT					/* Act as a memory editor */
};


/**** Define function prototypes that other modules may want to use ****/

extern	char *	Dprintf( int, char * control_string, ... );

extern	long	ProcTime( void );
extern	long	ProcClockSecond( void );
extern	 int	ProcTimeAfter( long time1, long time2 );
extern	long	ProcTimePlus( long time1, long time2 );
extern	void	ProcWait( long timer );

extern   int	RemoveConnectEntry( char * node );
extern	 int	AddConnectEntry( char * node );
extern	 int	FindConnectEntry( char * node );

extern	void	SigInt( int sig );
extern	void	SigHup( int sig );
extern	void	SigQuit( int sig );

extern   int	StartConnection( char * node );
extern   int	ConnectOnRequest( char * );
extern	 int	DisconnectReport( char * );
extern	 int	Connect( char * );
extern	 int	ConnectionReport( char * command );


#endif


/*********************************  EOF  ************************************/


